import de.metus.scripting.*
import de.metus.scripting.groovy.ScriptCommand
import groovy.jface.JFaceBuilder
import groovy.jface.MetusJFaceBuilder
import org.eclipse.swt.widgets.Label
import org.eclipse.swt.*
import org.eclipse.swt.layout.GridData
import org.eclipse.swt.graphics.Point
import org.eclipse.swt.widgets.Shell
import org.eclipse.swt.widgets.Display
import org.eclipse.swt.graphics.Color
import org.eclipse.swt.widgets.Button


Map<String, String> getGroovyScriptNameToDescription() {
	File scriptFolder = new File (appFolder.toString() + "\\scripts\\")
	List<File> filesInFolder = scriptFolder.listFiles()
	List<File> groovyScriptsInFolder = filesInFolder.findAll{it.getName().toLowerCase().endsWith(".groovy")}
	List<File> descriptionTextFilesInFolder = filesInFolder.findAll{it.getName().toLowerCase().endsWith(".txt")}
	Map<File, File> groovyScriptToDescriptionTxtFile = [:]

	groovyScriptsInFolder.each{
		String groovyFileName = it.getName()
		String groovyFileNameWithoutExtension = groovyFileName.substring(0, groovyFileName.size() - 7)
		boolean groovyScriptToDescriptionTxtFound = false

		for(File descriptionTextFileInFolder :  descriptionTextFilesInFolder) {
			String txtFileName = descriptionTextFileInFolder.getName()
			String txtFileNameWithoutExtension = txtFileName.substring(0, txtFileName.size() - 4)

			if(groovyFileNameWithoutExtension == txtFileNameWithoutExtension) {
				groovyScriptToDescriptionTxtFile.put(it, descriptionTextFileInFolder)
				groovyScriptToDescriptionTxtFound = true
				break
			}
		}
		if(!groovyScriptToDescriptionTxtFound) {
			groovyScriptToDescriptionTxtFile.put(it, null)
		}
	}

	Map<String, String> groovyScriptNameToDescription = groovyScriptToDescriptionTxtFile.collectEntries { key, value ->
		if (value == null)
			return [key.getName(), ""]
		else
			return [key.getName(), value.getText("UTF-8")]
	}

	descriptionTextFilesInFolder.each{
		String txtFileName = it.getName()
		String txtFileNameWithoutExtension = txtFileName.substring(0, txtFileName.size() - 4)
		boolean txtScriptToDescriptionTxtFound = false

		for(File groovyScriptInFolder :  groovyScriptsInFolder) {
			String groovyFileName = groovyScriptInFolder.getName()
			String groovyFileNameWithoutExtension = groovyFileName.substring(0, groovyFileName.size() - 7)

			if(txtFileNameWithoutExtension == groovyFileNameWithoutExtension) {
				txtScriptToDescriptionTxtFound = true
				break
			}
		}
		if(!txtScriptToDescriptionTxtFound) {
			console.println(txtFileNameWithoutExtension+" has no script!")
		}
	}

	return groovyScriptNameToDescription
}

Point computeRadioSize(String radioText) {
	def jface = new MetusJFaceBuilder()

	def shell = jface.shell('compute'){
		migLayout(layoutConstraints:"wrap 1", columnConstraints: "[100%]", rowConstraints: "[100%]")
		button(id: "radio", style: "RADIO", layoutData: "left", radioText)
	}

	metus.ui().center(shell)

	return jface.radio.getSize()
}

Point computeLabelSize(String labelText) {
	def jface = new MetusJFaceBuilder()

	def shell = jface.shell('compute'){
		migLayout(layoutConstraints:"wrap 1", columnConstraints: "[100%]", rowConstraints: "[100%]")
		label(id: "label", labelText)
	}

	metus.ui().center(shell)

	return jface.label.getSize()
}

Point computeTextSize(String textContent) {
	def jface = new MetusJFaceBuilder()

	def shell = jface.shell('compute'){
		migLayout(layoutConstraints:"wrap 1", columnConstraints: "[]", rowConstraints: "[]")
		text(id:"text", style:"WRAP, READ_ONLY, MULTI", textContent)
	}

	metus.ui().center(shell)

	return jface.text.getSize()
}

Point computeScriptSelectionSize(String labelText, List<String> scriptNames, int topPaddingScriptSelectionLabel, int bottomPaddingScriptSelectionLabel) {
	int height = 0
	int width = 0

	Point labelSize = computeLabelSize(labelText)
	height += labelSize.y
	width = Math.max(width, labelSize.x)

	for(String scriptName : scriptNames) {
		Point radioSize = computeRadioSize(scriptName)
		height += radioSize.y
		width = Math.max(width, radioSize.x)
	}

	width += topPaddingScriptSelectionLabel + bottomPaddingScriptSelectionLabel

	return new Point(width, height);
}

Point computeDescriptionTextSize(List<String> descriptionTexts) {
	int width = 0
	int height = 0

	for(String descriptionText : descriptionTexts) {
		Point descriptionTextSize = computeTextSize(descriptionText)
		width = Math.max(descriptionTextSize.x, width)
		height = Math.max(descriptionTextSize.y, height)
	}

	return new Point(width, height);
}

Point getOptimalScriptSelectionOrDescriptionTextSize(Point firstAreaRealSize, Point firstAreaExtraPaddingSize, Point secondAreaRealSize, Point secondAreaExtraPaddingSize, int areaMaxWidth, int areaMaxHeight, int areaMinWidth, int areaMinHeight) {
	int firstAreaTotalWidth = Math.max(firstAreaRealSize.x + firstAreaExtraPaddingSize.x, areaMinWidth)
	int firstAreaTotalHeight = Math.max(firstAreaRealSize.y + firstAreaExtraPaddingSize.y, areaMinHeight)
	int firstAreaWidthDiff = areaMaxWidth - firstAreaTotalWidth
	int firstAreaHeightDiff = areaMaxHeight - firstAreaTotalHeight

	int secondAreaTotalWidth = Math.max(secondAreaRealSize.x + secondAreaExtraPaddingSize.x, areaMinWidth)
	int secondAreaTotalHeight = Math.max(secondAreaRealSize.y + secondAreaExtraPaddingSize.y, areaMinHeight)
	int secondAreaWidthDiff = areaMaxWidth - secondAreaTotalWidth
	int secondAreaHeightDiff = areaMaxHeight - secondAreaTotalHeight

	int width
	if(firstAreaWidthDiff >= 0) {
		width = firstAreaTotalWidth
	} else if(secondAreaWidthDiff > 0) {
		width = Math.min(firstAreaTotalWidth, secondAreaWidthDiff + areaMaxWidth)
	} else {
		width = areaMaxWidth
	}

	int height = Math.min(areaMaxHeight, Math.max(firstAreaTotalHeight, secondAreaTotalHeight))

	return new Point(width, height)
}

Point getOptimalScriptSelectionSize(Point scriptSelectionRealSize, Point scriptSelectionExtraSize, Point descriptionTextRealSize, Point descriptionTextExtraSize, int areaMaxWidth, int areaMaxHeight, int areaMinWidth, int areaMinHeight) {
	return getOptimalScriptSelectionOrDescriptionTextSize(scriptSelectionRealSize, scriptSelectionExtraSize, descriptionTextRealSize, descriptionTextExtraSize, areaMaxWidth, areaMaxHeight, areaMinWidth, areaMinHeight)
}

Point getOptimalDescriptionTextSize(Point descriptionTextRealSize, Point descriptionTextExtraSize, Point scriptSelectionRealSize, Point scriptSelectionExtraSize, int areaMaxWidth, int areaMaxHeightt, int areaMinWidth, int areaMinHeight) {
	return getOptimalScriptSelectionOrDescriptionTextSize(descriptionTextRealSize, descriptionTextExtraSize, scriptSelectionRealSize, scriptSelectionExtraSize, areaMaxWidth, areaMaxHeightt, areaMinWidth, areaMinHeight)
}

String getDescriptionText(Map<File, File> groovyScriptToDescriptionTxt, File groovyScript) {
	if(groovyScriptToDescriptionTxt.containsKey(groovyScript) && groovyScriptToDescriptionTxt.get(groovyScript) != null)
		return groovyScriptToDescriptionTxt.get(groovyScript)
}



Shell createShell(Map<String, String> groovyScriptNameToDescription, Point optimalScriptSelectionSize, Point optimalDescriptionTextSize, Point optimalScriptSelectionScriptAreaSize, int maxShellWidth, int maxShellHeight, List<String> scripts, int topPaddingScriptSelectionLabel, int bottomPaddingScriptSelectionLabel, int buttonMinWidth, List<String> scriptDescription) {

	String scriptSelectionCompositeLayoutData =  "w " + optimalScriptSelectionSize.x + ", h " + optimalScriptSelectionSize.y + ", grow, shrink, wmin 40, hmin 40"
	String scriptSelectionRowContraints = topPaddingScriptSelectionLabel + "[fill]" + bottomPaddingScriptSelectionLabel + "[fill, grow, shrink]0"
	String scriptSelectionScriptAreaCompositeLayoutData = "w " + optimalScriptSelectionScriptAreaSize.x + ", h " + optimalScriptSelectionScriptAreaSize.y + ", grow, shrink, wmin 40, hmin 40"

	String descriptionTextCompositeLayoutData = "w " + optimalDescriptionTextSize.x + ", h " + optimalDescriptionTextSize.y + ", grow, shrink, wmin 40, hmin 40"

	String scriptSelectionRadioLinesRowConstraints = ""
	for (int i = 0; i < groovyScriptNameToDescription.size(); i++){
		scriptSelectionRadioLinesRowConstraints += "[]"
	}

	File scriptToExecute

	MetusJFaceBuilder jface = new MetusJFaceBuilder()
	Shell shell
	shell = jface.shell('App Entry Point Selection'){
		migLayout(layoutConstraints:"wrap 2", columnConstraints: "[fill, shrink, grow]10[fill, shrink, grow]", rowConstraints: "[shrink, grow]30[]0")

		composite(id:"scriptSelectionCompleteArea", layoutData: scriptSelectionCompositeLayoutData){
			migLayout(layoutConstraints:"wrap 1", columnConstraints: "0[fill, grow, shrink]0", rowConstraints: scriptSelectionRowContraints)
			label("Select an entry Point:")
			composite(id: "scriptListRadioAreaParent", layoutData: scriptSelectionScriptAreaCompositeLayoutData) {
				migLayout(layoutConstraints:"wrap 1", columnConstraints: "[]", rowConstraints: scriptSelectionRadioLinesRowConstraints)
				fillLayout()
				scrolledComposite (id:'scrolledCompositeScriptListRadioArea', style:'BORDER,H_SCROLL,V_SCROLL', expandHorizontal:true, expandVertical:true){
					composite(id: "scriptListRadioArea") {
						migLayout(layoutConstraints:"wrap 1", columnConstraints: "[]", rowConstraints: scriptSelectionRadioLinesRowConstraints)

						for(def e in groovyScriptNameToDescription) {
							String currentScriptNameWithExtension = e.getKey()
							String currentText = e.getValue()
							String currentScriptName = currentScriptNameWithExtension.substring(0, currentScriptNameWithExtension.size() - 7)
							button(id: currentScriptName, style: "RADIO", layoutData: "left", "  " + currentScriptName){
								onEvent('Selection') {
									jface.scriptDescription.setText(currentText)
					
									jface.scrolledCompositeDescriptionTextArea.setContent(jface.descriptionTextArea)
									jface.scrolledCompositeDescriptionTextArea.setMinSize(jface.descriptionTextArea.computeSize(SWT.DEFAULT, SWT.DEFAULT))
									
									scriptToExecute = new File(appFolder.toString() + "\\scripts\\" + currentScriptNameWithExtension)
								}
							}

							jface."$currentScriptName".setBackground(new Color(Display.getCurrent(), 255, 255, 255))
						}
					}
				}
			}
		}

		jface.scriptListRadioArea.setBackground(new Color(Display.getCurrent(), 255, 255, 255))
		jface.scrolledCompositeScriptListRadioArea.setContent(jface.scriptListRadioArea)
		jface.scrolledCompositeScriptListRadioArea.setMinSize(jface.scriptListRadioArea.computeSize(SWT.DEFAULT, SWT.DEFAULT))

		composite(id:"descriptionTextAreaParent", layoutData:descriptionTextCompositeLayoutData){
			migLayout(layoutConstraints:"wrap 1", columnConstraints: "[fill, grow, shrink]", rowConstraints: "[fill, grow, shrink]")
			fillLayout()
			scrolledComposite (id:'scrolledCompositeDescriptionTextArea', style:'FILL,BORDER,H_SCROLL,V_SCROLL', expandHorizontal:true, expandVertical:true){
				composite(id:'descriptionTextArea', layoutData: "wmin 40, hmin 40"){
					migLayout(layoutConstraints:"wrap 1", columnConstraints: "[]", rowConstraints: "[]")
					text(id:"scriptDescription", style:"WRAP, READ_ONLY, MULTI")
				}
			}
		}

		jface.scriptDescription.setText(groovyScriptNameToDescription.entrySet().toList().first().getValue())
		jface.scriptDescription.setBackground(new Color(Display.getCurrent(), 255, 255, 255))
		jface.descriptionTextArea.setBackground(new Color(Display.getCurrent(), 255, 255, 255))
		jface.scrolledCompositeDescriptionTextArea.setContent(jface.descriptionTextArea)
		jface.scrolledCompositeDescriptionTextArea.setMinSize(jface.descriptionTextArea.computeSize(SWT.DEFAULT, SWT.DEFAULT))


		int cancelButtonPadding = optimalScriptSelectionSize.x - buttonMinWidth
		String cancelButtonColumnConstraints =  "70%[" + "30%" + ", fill, shrink, grow]"

		composite(id:"a", layoutData: "wmin 10"){
			migLayout(layoutConstraints:"wrap 1", columnConstraints: cancelButtonColumnConstraints, rowConstraints: "[]")
			button("Cancel"){
				onEvent('Selection') {  shell.close()  }
			}
		}

		String runMacroButtonColumnConstraints = "0[30%, fill, shrink, grow]70%"

		composite(id:"b", layoutData: "WRAP, wmin 10"){
			migLayout(layoutConstraints:"wrap 1", columnConstraints: runMacroButtonColumnConstraints, rowConstraints: "[]")
			button("Run Macro"){
				onEvent('Selection') {
					shell.close()
					new ScriptCommand(scriptToExecute).run()
				}
			}
		}
	}


	metus.ui().center(shell)
	shell.doMainloop()
	return shell
}



void main(String labelText, int topPaddingScriptSelectionLabel, int bottomPaddingScriptSelectionLabel, int scriptSelectionOrDescriptionTextAreaWidth, int scriptSelectionOrDescriptionTextAreaHeight, int maxShellWidth, int maxShellHeight, int buttonMinWidth) {
	Map<String, String> groovyScriptNameToDescription = getGroovyScriptNameToDescription()
	List<String> scriptNames = groovyScriptNameToDescription.collect{
		key, value -> key.substring(0, key.size() - 7)
	}
	List<String> descriptionTexts = groovyScriptNameToDescription.collect{
		key, value -> value
	}

	Point scriptSelectionRealSize = computeScriptSelectionSize(labelText, scriptNames, topPaddingScriptSelectionLabel, bottomPaddingScriptSelectionLabel)
	Point scriptSelectionExtraSize = new Point(0, topPaddingScriptSelectionLabel + bottomPaddingScriptSelectionLabel)

	Point descriptionTextRealSize = computeDescriptionTextSize(descriptionTexts)
	Point descriptionTextExtraSize = new Point(0, 0)

	Point optimalScriptSelectionSize = getOptimalScriptSelectionSize(scriptSelectionRealSize, scriptSelectionExtraSize, descriptionTextRealSize, descriptionTextExtraSize, scriptSelectionOrDescriptionTextAreaWidth, scriptSelectionOrDescriptionTextAreaHeight, buttonMinWidth, 100)
	Point optimalDescriptionTextSize = getOptimalDescriptionTextSize(descriptionTextRealSize, descriptionTextExtraSize, scriptSelectionRealSize, scriptSelectionExtraSize, scriptSelectionOrDescriptionTextAreaWidth, scriptSelectionOrDescriptionTextAreaHeight, buttonMinWidth, 100)

	Point scriptSelectionLabelTextRealSize = computeLabelSize(labelText)
	Point optimalScriptSelectionScriptAreaSize = new Point(optimalScriptSelectionSize.x, optimalScriptSelectionSize.y - scriptSelectionLabelTextRealSize.y - topPaddingScriptSelectionLabel - bottomPaddingScriptSelectionLabel)

	createShell(groovyScriptNameToDescription,
			optimalScriptSelectionSize,
			optimalDescriptionTextSize,
			optimalScriptSelectionScriptAreaSize,
			maxShellWidth,
			maxShellHeight,
			scriptNames,
			topPaddingScriptSelectionLabel,
			bottomPaddingScriptSelectionLabel,
			buttonMinWidth,
			descriptionTexts)
}



int maxShellWidth = 760
int maxShellHeight = 400

int topPaddingScriptSelectionLabel = 15
int bottomPaddingScriptsSelectionLabel = 40
int scriptSelectionOrDescriptionTextAreaWidth = 360
int scriptSelectionTextAreaHeight = 270
int scriptSelectionOrDescriptionTextAreaHeight = 335
int buttonMinWidth = 100

def String labelText = "Select an Entry Point:"

if(appFolder.toString().contains("metusUnsavedDummyForAttachments")){
	metus.ui().openError(null, "No proper Model loaded!", "In order to run the Toolkit App please save the current Project or open a Project from the Server!")
	return
} else {
	main(labelText, topPaddingScriptSelectionLabel, bottomPaddingScriptsSelectionLabel, scriptSelectionOrDescriptionTextAreaWidth, scriptSelectionOrDescriptionTextAreaHeight, maxShellWidth, maxShellHeight, buttonMinWidth)
}

