import de.metus.scripting.*
import de.metus.scripting.Table.FormatType

import com.sun.xml.internal.bind.v2.schemagen.xmlschema.AttributeType

import de.metus.model.*
import de.metus.model.util.*


def formulaIterator(tableName, comparator) {
	metus.rootFolder().tableSub(tableName).each() { table ->
		table.attributes()
				// .findAll { attribute -> attribute.name == "Var_FCost_Ratio"}
				.findAll {it.getType() ==  Table.AttributeType.Formula}
				.each { attribute ->
					def formula = attribute.getFormula()
					def formulaAttr = attribute.attribute
					def formulaJs = attribute.getJsFormula()
					def formulaName = attribute.getName()
					def parentTableName =  attribute.getParentTable().name
					def isGroovyFormulaMissing = formula == "" || formula == null ;
					def isJsFormulaMissing = formulaJs == "" || formulaJs == null ;
					def qualifiedFormulaName = parentTableName + "." + formulaName;
					//console.println("Iterating " + qualifiedFormulaName);
					if (isGroovyFormulaMissing) {
						console.println(qualifiedFormulaName + ": missing Groovy Formula");
					} else if (isJsFormulaMissing) {
						console.println(qualifiedFormulaName + ": missing JS Formula");
					} else {
						
						// Berechnen der Formel fr alle Elemente
						attribute.table.elements().each { element ->
							comparator(table, attribute, element);
						}
					}
				}
	}
}

validResultsCount = 0;
failedResultsCount = 0;
totalValidResultsCount = 0;
totalFailedResultsCount = 0;

compareJSAndGroovyResult = { id, resultJs, resultGroovy ->
	// console.println("Comparing JS '" + resultJs + "' with '" + resultGroovy + "'")

	def isEqual;

	if (resultGroovy instanceof Number && resultJs instanceof Number) {
		def resultGroovyAsDouble = resultGroovy as double
		def delta = Math.abs(resultGroovyAsDouble-resultJs);
		isEqual = delta < 0.0000001;
	} else {
		isEqual = resultGroovy == resultJs;
	}

	if (!isEqual) {
		console.println( id + ": Groovy = '" + resultGroovy + "', JavaScript = '" + resultJs + "'")
		failedResultsCount +=1
	} else {
		validResultsCount +=1
	}
	

}

createId = { table, attribute, element ->
	def id = table.getName() + "." + attribute.getName() + "." + element.getName() + "=" + element.element.generatedId
	// console.println("ID: " + id);
	return id;
}

saveFormulaValue = { table, attribute, element ->
	def formulaModelAttribute = attribute.attribute
	def result = formulaModelAttribute.mGetFromFormula(element.element)
	def id = createId(table, attribute, element);
	def stringResult = ""+result
	def isErrorInJsFormula = stringResult.startsWith("#Error") || stringResult.startsWith("#TypeError");
	if (isErrorInJsFormula) {
		// Dont log whole Error to reduce output length
		result = "Error in Js Formula!"
	}
	values[id] = result;
	//console.println("Saving: " + id + " -> " + result);
}

compareFormulaValueWithSavedValue = { table, attribute, element ->
	def formulaModelAttribute = attribute.attribute
	def result = formulaModelAttribute.mGetFromFormula(element.element)
	def id = createId(table, attribute, element);
	compareJSAndGroovyResult(id, values[id], result);
}

values = [:]

profiledRun = {	tableNames ->
	console.println("Start profiling ...")
	metus.startFormelProfiling();
	tableNames.each{ tableName ->formulaIterator(tableName, saveFormulaValue) };
	def result = metus.endFormelProfiling();
	console.println(" ... profiling done")
	return result;
}

compareFormulaValues = { tableName ->
	console.println("Collecting JavaScript values")
	FormulaEvaluator.getEvaluatorSubject().setLanguageMode(EvaluatorSubject.ScriptingLanguageMode.JavaScript);
	formulaIterator(tableName, saveFormulaValue)
	console.println("Collecting Groovy values and compare to JavaScript values")
	FormulaEvaluator.getEvaluatorSubject().setLanguageMode(EvaluatorSubject.ScriptingLanguageMode.Groovy);
	formulaIterator(tableName, compareFormulaValueWithSavedValue)
	console.println("${tableName}: found ${validResultsCount} equal and ${failedResultsCount} differing formula results." )
	totalValidResultsCount += validResultsCount;
	totalFailedResultsCount += failedResultsCount;
	validResultsCount = 0;
	failedResultsCount = 0;
}

profiledJSRun = { tableNames ->
	FormulaEvaluator.getEvaluatorSubject().setLanguageMode(EvaluatorSubject.ScriptingLanguageMode.JavaScript);
	return profiledRun(tableNames);
}

profiledGroovyRun = { tableNames ->
	FormulaEvaluator.getEvaluatorSubject().setLanguageMode(EvaluatorSubject.ScriptingLanguageMode.Groovy);
	return profiledRun(tableNames);
}

sumAndRenameFormulaExecTimesTable = { tableAppendix ->
	def selfTimeMillis = 0
	metus.rootFolder().tableSub("_FORMULA_EXECUTION_TIMES").each { table ->
		table.elements().each { row ->
			selfTimeMillis += row.SelfTimeMillis	
		}
		table.elementType.setName("EXEC_TIMES_" + tableAppendix)
	}
	return selfTimeMillis;
}


runJavaScriptAndGroovyOnTableWithStats = { table ->
	def selfTimeMillisJava = 0
	def selfTimeMillisGroovy = 0
	if (true) {
		console.println("Run JavaScript OnTableWithStats: " + table)
		def timeMillisJava = profiledJSRun([table]);
		selfTimeMillisJava = sumAndRenameFormulaExecTimesTable(table + "_JavaScript");
	}
	if (true) {
		console.println("Run Groovy OnTableWithStats: " + table)
		def timeMillisGroovy = profiledGroovyRun([table]);
		selfTimeMillisGroovy = sumAndRenameFormulaExecTimesTable(table + "_Groovy");
	}
	return [selfTimeMillisJava, selfTimeMillisGroovy];
}

createResultTable = { 
	if (metus.rootFolder().tableExists("Admin", "Result")) {
		metus.rootFolder().table("Admin", "Result").delete()
	}
	def resultTable = metus.rootFolder().createTable("Admin", "Result")
	resultTable.createAttribute("selfTimeMillisGroovy", de.metus.scripting.Table.AttributeType.String, FormatType.Number, "#0")
	resultTable.createAttribute("selfTimeMillisJavascript", de.metus.scripting.Table.AttributeType.String, FormatType.Number, "#0")
	resultTable.createAttribute("factorJavascript", de.metus.scripting.Table.AttributeType.String, FormatType.Number, "#0,00")
	return resultTable
}

runJavaScriptAndGroovyOnTablesWithStats = { tableNames ->
	def resultTable = createResultTable();
	tableNames.each { tableName ->
		def stats = runJavaScriptAndGroovyOnTableWithStats(tableName)
		def resultRow = resultTable.createElement(tableName)
		resultRow.set("selfTimeMillisJavascript", stats[0] as int)
		resultRow.set("selfTimeMillisGroovy", stats[1] as int) 
		if (stats[1] as int != 0) {
			resultRow.set("factorJavascript", (stats[0]/stats[1]).toString().replace(".", ","))
		}
		
	}
}

def allTableNames = metus.allTables().collect{it.name};

// runJavaScriptAndGroovyOnTableWithStats(["Var_Component","Component"]) }
// profiledJSRun();
// profiledGroovyRun();
// compareFormulaValues("Function_3");


// compareAllFormulaValues
allTableNames.each { tableName -> 
	console.println("--------------------------------------------- Evaluating Table " + tableName + " ---------------------------------------------")
	compareFormulaValues(tableName)
}
console.println("Overall there were " + totalValidResultsCount + " equal formula results and " + totalFailedResultsCount + " differing results!")

// allTableNames.each{tableName -> compareFormulaValues(tableName)}

// Exec all Formulas in one Language and create a table with the results in Admin/_FORMULA_EXECUTION_TIMES 
// Profiling in settings has to be turned on!!
// profiledJSRun(allTableNames)
// profiledGroovyRun(allTableNames)

// Exec all Formulas of one or more tables both in JS and Groovy and create result tables in Admin
// runJavaScriptAndGroovyOnTablesWithStats(["Var_Component"])

// reset global mode
FormulaEvaluator.getEvaluatorSubject().setLanguageMode(EvaluatorSubject.ScriptingLanguageMode.Mixed);

