import de.metus.scripting.*

/**
 * @author Patrick Bernhard
 * 18/01/2019
 *
 * This script searches for elements with equal names (the elements must be in the same table)
 * either in one or more specified tables or in all tables
 * and prints the result in the console.
 */

/*
 * Define here, which specific table should be searched for elements with equal names.
 * Non existing tables will be mentioned in the console but do not lead to a crash.
 * Also see "searchAllTables".
 */
List<String> tableNames = ["Variant_driver", "TestTable", "NonExistingTable"]

/*
 * If this is set to true every table in the model will be searched for elements with equal names.
 * "tableNames" is ignored.
 */
boolean searchAllTables = false





// Code -------------------------------------

if(searchAllTables) {
	tableNames = metus.rootFolder().tablesSub().collect{it.getName()}
}

for(String tableName : tableNames) {
	if(!metus.rootFolder().tableExistsSub(tableName)) {
		console.println(" - The given table \"" + tableName + "\" does not exist in the model!\n\n")
		continue;
	}
	Table table = metus.rootFolder().tableSub(tableName)
	List<String> duplicatedNames = []
	Map<String, Integer> numOccurrencesByName = [:]

	for(Element element : table.elements()) {
		if(numOccurrencesByName.containsKey(element.name)) {
			int numberNow = numOccurrencesByName.get(element.name)
			numOccurrencesByName.remove(element.name)
			numOccurrencesByName.put(element.name, numberNow + 1)

			if(!duplicatedNames.contains(element.name))
				duplicatedNames.add(element.name)
		} else {
			numOccurrencesByName.put(element.name, 1)
		}
	}

	if(duplicatedNames.size() == 0) {
		console.println(" - No multiple equal names in table \"" + tableName + "\"\n\n")
	} else {
		console.println(" - Multiple equal names found in table \"" + tableName + "\":")
		for(String name : duplicatedNames) {
			if(name == "")

				console.println(" \t* " + numOccurrencesByName.get(name) + "x \"" + name + "\" (= empty name)")
			else
				console.println(" \t* " + numOccurrencesByName.get(name) + "x \"" + name + "\" (without quotes)")
		}
		console.println("\n")
	}
}