import java.util.List;

import de.metus.scripting.*;
import de.metus.scripting.AttributeDescription;
import de.metus.tools.TableAndAttributeSelectionDialog.*;
import de.metus.tools.AttributeSelectionDialog.*;
import groovy.jface.MetusJFaceBuilder;
import org.eclipse.swt.widgets.Shell;

jface = new MetusJFaceBuilder()
def shell
Chart chart = metus.activeChart();
if (chart == null){
	metus.ui().openError(shell, "Please open a chart!", "You need to open a chart before executing this Script!")
	return
}



def tables = chart.tables();

shell = jface.shell('Selection'){
	migLayout(layoutConstraints:"wrap 1", columnConstraints: "[grow, shrink]", rowConstraints: "[grow, shrink]10[grow, shrink]")

	composite(id:'composite1', layoutData:"grow, shrink, wmin 10"){
		migLayout(layoutConstraints:"wrap 2", columnConstraints: "[fill][grow, shrink, fill]")
		
		label("Table 1:")
		combo(id: "table1", style: "READ_ONLY"){
			comboViewer(input: bind(model:tables, modelProperty:"name")){
				selectionChangedListener(){ updateAttr1() }
			}
		}

		label("Attribute 1:")
		combo(id: "attr1", style: "READ_ONLY"){
		}
		label("Attribute 2:")
		combo(id: "attr2", style: "READ_ONLY"){
		}
		label("Attribute 3:")
		combo(id: "attr3", style: "READ_ONLY"){
		}
		label("Attribute 4:")
		combo(id: "attr4", style: "READ_ONLY"){
		}
		label("Attribute 5:")
		combo(id: "attr5", style: "READ_ONLY"){
		}
	}
	composite(id:'composite2', layoutData:"grow, shrink, wmin 10"){
		migLayout(layoutConstraints:"wrap 4", columnConstraints: "0[25%, shrink, grow, fill]0[25%, shrink, grow, fill]0[25%, shrink, grow, fill]0[25%, shrink, grow, fill]", rowConstraints: "[shrink, grow]")

		label("")
		
		button("Sort", default:true){
			onEvent('Selection'){
				List<String> attributeName = [];
				for (int i=1; i<=4; i++){
					String step = jface.getVariable("attr${i}").getText();
					if (step!=""){
						attributeName += step;
					}
				}
				result = attributeName;
				console.println("Selected Attributes: " + result);
				if (attributeName.empty){
					metus.ui.openInformation(null, "Attribute Selection", "Please select at least one attribute to sort by!");
				} else {
					table = jface.table1.getText();
					execute(shell, chart, table, attributeName)
				}
			}
		}
		
		button("Cancel"){
			onEvent('Selection') { shell.close() }
		}
		
		label("")
	}
}
updateAttr1()
metus.ui().center(shell)
shell.doMainloop()
void updateAttr1(){
	if(jface.table1.getText() == ""){
		jface.attr1.setEnabled(false);
		jface.attr2.setEnabled(false);
		jface.attr3.setEnabled(false);
		jface.attr4.setEnabled(false);
		jface.attr5.setEnabled(false);
		jface.attr1.removeAll()
		jface.attr2.removeAll()
		jface.attr3.removeAll()
		jface.attr4.removeAll()
		jface.attr5.removeAll()
	}else{
		jface.attr1.removeAll()
		jface.attr2.removeAll()
		jface.attr3.removeAll()
		jface.attr4.removeAll()
		jface.attr5.removeAll()
		jface.attr1.setEnabled(true);
		jface.attr2.setEnabled(true);
		jface.attr3.setEnabled(true);
		jface.attr4.setEnabled(true);
		jface.attr5.setEnabled(true);
		for(def attr: metus.rootFolder().tableSub(jface.table1.getText()).attributes().name){
			jface.attr1.add(attr)
			jface.attr2.add(attr)
			jface.attr3.add(attr)
			jface.attr4.add(attr)
			jface.attr5.add(attr)
		}
	}
}

void execute(shell, chart, table, attributes){

	ChartLevel level = chart.level(table);
	List<VisualElement> visualElements = level.visualElements();
	// sort by two attributes
	visualElements.sort { a,b->
		for (String att: attributes) {
			// not equal -> return result, otherwise loop to next level
			if (a.element.get(att) != b.element.get(att)){
				return a.element.get(att)<=>b.element.get(att);
			}
		}
		return 0;
	}

	// layout according to sort order
	int ypos = 100;
	for (VisualElement ve:visualElements){
		ve.setYPos(ypos);
		ypos += ve.getHeight() + 5;
	}
	shell.close()
}





